module DrillMudsThermalProps

import Interpolations: linear_interpolation
import ForwardDiff: gradient
import LinearAlgebra: dot
import JLD2: jldopen
import Unitful: @u_str, uconvert, upreferred, Quantity, ustrip
import Integrals: IntegralProblem,solve,QuadGKJL
import XLSX: openxlsx, writetable!, rename!, CellRef, addsheet!
import InteractiveUtils: subtypes
import CSV: write
import Tables: table
using SnoopPrecompile


include("Erros.jl")
include("Materials.jl")
include("EquationsOfState.jl")

include("Correlations.jl")

Fluids_paths=joinpath(@__DIR__,"Fluids")
Fluids_files=joinpath.(Fluids_paths,readdir(Fluids_paths))
# println((fs=Fluids_files,)) #DEBUG
include.(Fluids_files)

Solids_paths=joinpath(@__DIR__,"Solids")
Solids_files=joinpath.(Solids_paths,readdir(Solids_paths))
include.(Solids_files)




include("Mixture.jl")
Trans_Mix_rules_paths=joinpath(@__DIR__,"TransportPropsMixtureRules")
Trans_Mix_rules_files=joinpath.(Trans_Mix_rules_paths,readdir(Trans_Mix_rules_paths))
include.(Trans_Mix_rules_files)



include("InputTableParser.jl")
include("Exporter.jl")




@precompile_setup begin

    @precompile_all_calls begin
        println("Precompilation on DrillMudsThermalProps")
        wat=Water()
        br950=BrineNaCl(9.5)
        brStd=BrineNaCl()
        hex=Hexadecene()

        cp=Cp(2E6,300,hex)
        cp=Cp(2E6,300,br950)
        beta=therm_expand(2E6,300,hex)
        kappa=compress(2E6,300,hex)
        for F in subtypes(Fluid)
            try
                f=F()
                rho(101325.0,298.15,f)
                Cp(50.0E6,320.0,f)
                therm_cond(50.0E6,320.0,f)
                therm_expand(50.0E6,320.0,f)
                compress(50.0E6,320.0,f)
                visc(50.0E6,320.0,f)
            catch
                continue
            end
        end
        FluidComp=(
        Componente=[
            :Olefina,
            :Brine_NaCl,
            :Barita
        ],
        conc=[
            0.58,
            0.39,
            missing
        ],
        unit=[
            "bbl/bbl",
            "bbl/bbl",
            "QSP"
        ],
        aditive=[
            false,
            false,
            false
        ]
    )

    fluid=solicitation_table_parser(FluidComp,9.5,5.0u"l")

    drill=DrillFluid(collect(Iterators.flatten(fluid.fluid_content)),collect(Iterators.flatten(fluid.solid_content)),fs=collect(Float64,Iterators.flatten(fluid.fcomp)))

    Cp(10.0E6,300.0,drill)
    rho(10.0E6,300.0,drill)
    therm_expand(10.0E6,300.0,drill)
    compress(10.0E6,300.0,drill)
    therm_cond(10.0E6,300.0,drill)
    visc(10.0E6,300.0,drill)

    end

end








end
