export generate_JLD2,
        generate_XLSX,
        generate_csv_tables



function generate_JLD2(fname,props_dics,Ts,Ps,fluid;include_std=true)

    pair_PT_table=[(P=P,T=T) for P in Ps, T in Ts]
    pair_PT=[pair_PT_table...]

    len_PT=length(pair_PT)
    jump_T=length(Ts)
    size_PT=size(pair_PT_table)

    props_keys=keys(props_dics)



    props_lines=Dict{Union{Symbol,String},Array{Float64}}(
        [k => zeros(Float64,len_PT) for k in props_keys]
    )

    props_tabs=Dict{Union{Symbol,String},Array{Float64}}(
        [k => zeros(Float64,size_PT) for k in props_keys]
    )



    for k in props_keys
        for (i, P) in enumerate(Ps)
            for (j, T) in enumerate(Ts)
                prop_calc = props_dics[k](P, T, fluid)

                props_tabs[k][i, j] = prop_calc

                props_lines[k][(i-1)*jump_T+j] = prop_calc

            end
        end
    end




    jldopen(fname,"w") do file
        file["fluid"]=fluid

        file["inputs/Ts"]=Ts
        file["inputs/Ps"]=Ps
        file["inputs/pairs"]=pair_PT
        file["inputs/table"]=pair_PT_table

        file["inputs/len_data"]=len_PT
        file["props"]=[k for k in props_keys]

        file["data/linear"]=props_lines
        file["data/tabular"]=props_tabs


    end


    return true

end


function generate_XLSX(fname,props_dics,Ts,Ps,fluid::Union{Fluid,DrillFluid},units;T_unit="K",P_unit="Pa")

    println((P_unit=P_unit,T_unit=T_unit))
    if T_unit == "K"
        Ts_SI=Ts
    elseif T_unit == "°C"
        Ts_SI=Ts.+273.15
    else
        error("Unit conversion not implemented! Check if unit string is correct.")
    end
    if P_unit == "Pa"
        Ps_SI=Ps
    elseif P_unit == "bar"
        Ps_SI = Ps.*1.0E5
    else
        error("Unit conversion not implemented! Check if unit string is correct.")
    end

    pair_PT_table=[(P=P,T=T) for P in Ps_SI, T in Ts_SI]
    pair_PT=[pair_PT_table...]

    size_PT=size(pair_PT_table)
    props_keys=keys(props_dics)
    props_tabs=Dict{Union{Symbol,String},Array{Float64}}(
        [k => zeros(Float64,size_PT) for k in props_keys]
    )


    for k in props_keys
        for (i, P) in enumerate(Ps_SI)
            for (j, T) in enumerate(Ts_SI)
                prop_calc = props_dics[k](P, T, fluid)
                props_tabs[k][i, j] = prop_calc
            end
        end
    end


    openxlsx(fname,mode="w") do xf

        for (i,k) in enumerate(props_keys)
            if i ==1
                sh=xf[1]
                rename!(sh,k)
            else
                addsheet!(xf,k)
                sh=xf[i]
            end
            # rename!(sh,k)
            sh["A1"] = "Unit:"
            sh["B1"] = "["*units[k]*"]"
            sh["C2"] = "T ["*T_unit*"]"
            sh["B3"] = "P ["*P_unit*"]"
            # writetable!(sh,Ts',[_ for T in Ts],anchor_cell=CellRef("D2"))
            # writetable!()
            sh["D2", dim = 2] = Ts
            sh["B4", dim = 1] = Ps

            sh["D4"] = props_tabs[k]
        end
    end

end



function generate_csv_tables(fpath,props_dics,Ts,Ps,fluid::Union{Material,DrillFluid}; include_std = true)

    try
        mkpath(fpath)
    catch e
        println("Erro=",e)
    end


    pair_PT_table=[(P=P,T=T) for P in Ps, T in Ts]
    pair_PT=[pair_PT_table...]

    len_PT=length(pair_PT)
    jump_T=length(Ts)
    size_PT=size(pair_PT_table)

    props_keys=keys(props_dics)


    props_tabs=Dict{Union{Symbol,String},Array{Float64}}(
        [k => zeros(Float64,size_PT) for k in props_keys]
    )

    base_name = "temperature_equivLIQUID_"
    base_header = "pressure/temperature"
    ext = ".csv"

    for k in props_keys
        for (i,P) in enumerate(Ps)
            for (j,T) in enumerate(Ts)
                prop_calc = props_dics[k](P,T,fluid)
                props_tabs[k][i, j] = prop_calc
            end
        end
    end
    tab = let
        tab_p = Dict()
        for k in props_keys
            p_tab = props_tabs[k]

            tab = Array{Any}(undef,(size(p_tab) .+ [1,1])...)

            tab[1,1] = base_header
            for (j,T) in enumerate(Ts)
                tab[1,j+1] = T
                for (i,P) in enumerate(Ps)
                    tab[i+1,j+1] = p_tab[i,j]
                end
            end
            for (i,P) in enumerate(Ps)
                tab[i+1,1] = P
            end
            tab_p[k] = tab
            file = joinpath(fpath,base_name*k*ext)
            tab_save = table(tab)
            write(file, tab_save, writeheader = false)
        end
        tab_p
    end
    return props_tabs, tab

end


function generate_csv_tables(fpath,Ts,Ps,fluid::Union{Material,DrillFluid}; include_std = true)

    props_dics = Dict(
        "compressibility" => compress,
        "conductivity" => therm_cond,
        "cp" => Cp,
        "density" => rho,
        "expansivity" => therm_expand,
        "viscosity" => visc,
        "mass_fraction" =>  (P,T,f) -> 1.0
    )

    return generate_csv_tables(fpath,props_dics,Ts,Ps,fluid,include_std=include_std)

end
