export Diesel


struct Diesel <: Fluid

    name::Symbol
    ρ::Float64
    cₚ::Float64

    Cp_params::Array{Float64,1}

    μ::Float64
    k::Float64

    TammannTait_params::NamedTuple

    k_param::Array{Float64,1}


    function Diesel()

        ## TammannTait_params fited from data...
        TTps=(
            E=0.03771958930452012,
            F0=3.3461951483140326E8,
            F1=-1.433460783117696E6,
            F2=1574.093836162755,
            )

        k_par=[
            0.20325750158739658
            -0.00015993688116013925
            -1.674357963605727E-7
            1.3664016661420837E-10
            8.932421437365109E-10
            -3.7843792123133525E-12
            1.2371197006465366E-14
            -1.0663750918441963E-17
            1.495414890313975E-18
            -1.3331721240387674E-20
            2.780725267691227E-23
            -2.8494247816891586E-26
        ]

        Cp_params=[
            -18.63800802352584
            6.54953482039806
            0.0047207592035462265
            -1.271237954912546E-5
            2.0457529374786374E-6
            -2.08861974040241E-8
            6.364576798066197E-11
            -8.335723847819817E-14
            -1.325717265100147E-14
            1.3389908364322073E-16
            -4.337856900784463E-19
            5.90289539694246E-22
        ]

        new(
            :Diesel,
            772.0149732871278,
            1956.8082715522821,
            Cp_params,
            3.0E-3,
            0.14659680680876513,
            TTps,
            k_par
        )

    end
end



rho_stdP(T,fluid::Diesel) = 902.7833869284138 + -0.44086212585786866T + 0.00020154456586086802*T^2 + -8.555125405403957e-7*T^3


function rho(P,T,fluid::Diesel,model::TT)

    rho_stdP=DrillMudsThermalProps.rho_stdP(T,fluid)

    (E,F0,F1,F2)=fluid.TammannTait_params
    num=P+F0+F1*T+F2*T*T
    den=101325.0+F0+F1*T+F2*T*T

    return rho_stdP/(1-E*log(num/den))
end

rho(P,T,fluid::Diesel) = rho(P,T,fluid,TT())


function Cp(P,T,fluid::Diesel,model::CpPStd)
    return -5.950871439867365+6.451375949823255T+0.004955583701780361T*T-1.2859613986876857E-5T*T*T
end

function Cp(P,T,fluid::Diesel,model::CpPVar)
    Cp0=Cp(P,T,fluid,cp0)
    f(Ps,args) = (therm_expand(Ps,T,fluid)^2+dbetadT(Ps,T,fluid))/rho(Ps,T,fluid)

    prob=IntegralProblem(f,101325.0,P,[])
    # println((Cp0=Cp0,bla=f(P,[T,fluid]),prob=prob))
    sol=solve(prob,QuadGKJL())#,nout=1,batch=0)
    # println((P=P,T=T))

    return Cp0-T*sol[1]
end


function Cp(P,T,fluid::Diesel,model::CpFit)
    out=0
    l=1
    params=fluid.Cp_params
    for i in range(0,2) # hard code
        for j in range(0,3)
            out+=params[l]*P^(i)*T^(j)
            l+=1
        end
    end
    return out
end


Cp(P,T,fluid::Diesel) = Cp(P,T,fluid,cpF)


function therm_cond(P,T,fluid::Diesel)
    out=0
    l=1
    params=fluid.k_param
    for i in range(0,2)#hard code, maybe something better on the future
        for j in range(0,3)
            out+=params[l]*P^(i)*T^(j)
            l+=1
        end
    end
    return out
end
