export Tridecane

struct Tridecane <: Fluid

    name::Symbol
    ρ::Float64
    cₚ::Float64
    μ::Float64
    k::Float64

    MM::Float64

    Tc::Float64

    TammannTait_params::NamedTuple

    k_param::Array{Float64,1}

    function Tridecane()
        MM=184.3614

        Tc=676.0

        TTps=(
            E=0.0886,
            F0=-81.898,
            F1=68.507,
            F2=4.247
        )


        k_par=[
            0.19519919129960064
            -0.00014757660370439215
            -2.1396569861216396e-7
            1.96514325174946E-10
            7.004007910709781E-10
            -2.438729042455057E-12
            9.09190970639116E-15
            -7.919303443022968E-18
            8.140157288443501E-19
            -8.73313082908299E-21
            1.753164176224094E-23
            -2.172958582374802E-26
        ]



        new(
            :nTridecane,
            760.15,
            100000000,
            3.0E-3,
            100000000,
            MM,
            Tc,
            TTps,
            k_par
        )

    end
end


function rho_stdP_nTridecane(T,fluid::Tridecane)
    return 937.2316-0.5296656T-2.945717E-4*T^2
end


function rho(P,T,fluid::Tridecane,EOS::TT)
    rho0=rho_stdP_nTridecane(T,fluid)

    (E,F0,F1,F2)=fluid.TammannTait_params
    Tc=fluid.Tc
    Tr=Tc/T
    num=1E-6P+F0+F1*Tr+F2*Tr*Tr
    den=101325.0E-6+F0+F1*Tr+F2*Tr*Tr

    return rho0/(1-E*log(num/den))
end


rho(P,T,fluid::Tridecane) = rho(P,T,fluid,TT())


function Cp(P,T,fluid::Tridecane,model::CpPStd)
    return (1.665948+7.684213E-4T+3.203705E-6T^2+9.873833E-10T^3)*1E3
end

Cp(P,T,fluid::Tridecane) = Cp(P,T,fluid,cpP)


function therm_cond(P,T,fluid::Tridecane,model::kYaws)
    A=-1.6564
    B=0.9372
    C=675.80
    log_lamb=A+B*(1.0-T/C)^(2/7)
    return 10^log_lamb
end


function therm_cond(P,T,fluid::Tridecane,model::kFit)
    out=0
    l=1
    params=fluid.k_param
    for i in range(0,2)#hard code, maybe something better on the future
        for j in range(0,3)
            out+=params[l]*P^(i)*T^(j)
            l+=1
        end
    end
    return out
end


therm_cond(P,T,fluid::Tridecane) = therm_cond(P,T,fluid,kfit)
