@testset "Water tests." begin
    using CoolProp

    water = Water()

    Ts = collect(range(4.0,100.0,5).+273.15)
    Ps = collect(range(101325.0,50E6,7))

    PTs = [[P,T] for P in Ps, T in Ts]

    @testset "Water Properties @ P=$(P) Pa and T=$(T) K." for P in Ps, T in Ts
        @testset "Density" begin
            @test rho(P,T,water) ≈ PropsSI("D","P",P,"T|liquid",T,"Water") rtol=5E-2
        end
        @testset "Specific Heat" begin
            @test Cp(P,T,water) ≈ PropsSI("C","P",P,"T|liquid",T,"Water")  rtol=5E-2
        end
        @testset "Viscosity" begin
            @test visc(P,T,water) ≈ PropsSI("V","P",P,"T|liquid",T,"Water") rtol=1E-1
        end
        @testset "Thermal Conductivity" begin
            @test therm_cond(P,T,water) ≈ PropsSI("L","P",P,"T|liquid",T,"Water") rtol=1E-1
        end
        @testset "Enthalpy" begin
            @test enthalpy(P,T,water) - enthalpy(101325.0,298.15,water) ≈ PropsSI("H","P",P,"T|liquid",T,"Water")-PropsSI("H","P",101325.0,"T|liquid",298.15,"Water") rtol = 2E-1
        end
    end
end
